package studenttable;

import javax.swing.table.*;
import java.sql.*;
import java.util.*;

/**
 * @author noah_singer
 */
public class DBTableModel extends AbstractTableModel {
	private ResultSet data;
	private ResultSetMetaData meta;
	private TreeMap hints;
	private Connection db;

	private boolean editable = true;
	
	public DBTableModel( String host, String database, String user, String password, String table )
		throws ClassNotFoundException, SQLException {

		Class.forName("com.mysql.jdbc.Driver");
		db = DriverManager.getConnection("jdbc:mysql://"
																		 + host
																		 + "/"
																		 + database,
																		 user,
																		 password);

		Statement statement = db.createStatement( ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE );
		data = statement.executeQuery("select * from " + table);

		meta = data.getMetaData( );
		hints = new TreeMap( );
	}

	public Object getValueAt( int row, int col ) {
		try {
			data.absolute(row + 1);

			String field = (String)hints.get( meta.getColumnName(col + 1) );
			if( field != null ) {
				String tableName = getForeignTableName( meta.getColumnName( col + 1) );

				Statement statement = db.createStatement( );

				ResultSet foreignData = statement.executeQuery( "select " + field + " from " + tableName + " where id=" + data.getInt(meta.getColumnName(col + 1)) );
				foreignData.absolute( 1 );
				return foreignData.getObject( 1 );
			} else {
				return data.getObject(col + 1);
			}
		} catch( SQLException se ) {
			System.err.println( se );
			return "N/A";
		}
	}

	public void setValueAt( Object value, int row, int col ) {
		try {
			data.absolute( row + 1 );
			data.updateObject(col+1, value);
			data.updateRow();
		} catch( SQLException se ) {
			System.err.println( se );
		}
	}

	public int getColumnCount( ) {
		try {
			return meta.getColumnCount( );
		} catch( SQLException se ) {
			System.err.println( se );
			return 0;
		}
	}

	public int getRowCount( ) {
		try {
			data.last( );
			return data.getRow( );
		} catch( SQLException se ) {
			System.err.println( se );
			return 0;
		}
	}

	public String getColumnName( int col ) {
		try {
			String hint = (String)hints.get( meta.getColumnName(col + 1) );
			if( hint != null ) {
				return formattedColumnName(getForeignTableName(meta.getColumnName(col + 1)));
			} else {
				return formattedColumnName(meta.getColumnName(col + 1));
			}
		} catch( SQLException se ) {
			System.err.println( se );
			return "N/A";
		}
	}

	public boolean isCellEditable( int row, int col ) {

		try {
			if( editable ) {
				if( meta.getColumnName(col + 1).equals( "id" ) ) {
					return false;
				} else {
					return true;
				}
			} else {
				return false;
			}
		} catch( SQLException se ) {
			System.err.println( se );
			return false;
		}
	}

	public void setEditable( boolean e ) {
		editable = e;
	}

	public boolean isEditable( ) {
		return editable;
	}

	public void addForeignKeyHint( String colName, String hint ) {
		hints.put( colName, hint );
	}

	public void removeForeignKeyHint( String colName ) {
		hints.remove( colName );
	}

	private String getForeignTableName( String fKey ) {
		//class_id -> classes
		//student_id -> students
		String tableName = fKey.substring( 0, fKey.length() - 3 );
		if( tableName.endsWith("s") ) {
			tableName += "es";
		} else {
			tableName += "s";
		}

		return tableName;
	}

	private String formattedColumnName( String colName ) {
		String [ ] parts = colName.split("_");
		String returnValue = "";

		for( int x = 0 ; x < parts.length ; x++ ) {
			returnValue += capitalize( parts[ x ] ) + " ";
		}

		return returnValue;
	}

	private String capitalize( String word ) {
		String firstChar = String.valueOf( word.charAt(0) );
		firstChar = firstChar.toUpperCase( );
		String theRest = word.substring(1, word.length( ) );
		return firstChar + theRest;
	}
}
